/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file main.h
 *
 * @type core
 *
 * @description
 *
 * The main core system, as its name suggests, is the main system. It handles
 * standard memory allocation, WIMP polling, event routing, memory transfers
 * and other key functions.
 *
 * The memory allocation is done via a similar system to the malloc-style
 * calls, renamed rmalloc, rfree and rcalloc (note there is no rrealloc).
 * All ROV functions use these instead of the standard malloc-style calls,
 * as the memory pool can be extended from the WIMP, and also freed back to
 * the WIMP if possible.
 *
 * Other core systems and subsystems interface with the main core's event
 * routing, so this really has to be used with ROV functions.
 */

#ifndef __rov_main
#define __rov_main


#ifdef __cplusplus
extern "C"
{
#endif

#include <stdlib.h>

#ifndef BOOL
/**
 * Definition of a BOOL.
 */
#define BOOL int

/**
 * Definition of TRUE.
 */
#define TRUE (1)

/**
 * Definition of FALSE.
 */
#define FALSE (0)
#endif

/**
 * Definition of a canvas.
 * @note It is cast as an integer pointer, because certain functions need to
 * have access to the information contained within. Applications should not do
 * this (this may be enforced at a later date).
 */
typedef int *canvas;

/**
 * Definition of an icon.
 */
typedef void *icon;

/**
 * Definition of a menu.
 */
typedef void *menu;

/**
 * Structure of a date string.
 */
typedef struct {
  char d[5];
} date_str;

/**
 * Structure of a WIMP message block.
 * @field size the size of the WIMP message.
 * @field sender the task handle of the sender.
 * @field my_ref the application's reference.
 * @field your_ref the sender's reference.
 * @field action the message action.
 * @field data.chars the information expressed as a character array.
 * @field data.words the information expressed as an integer array.
 * @field data.xfer.w the WIMP window handle for a transfer.
 * @field data.xfer.i the WIMP icon handle for a transfer.
 * @field data.xfer.x the (screen) x coordinate for a transfer.
 * @field data.xfer.y the (screen) y coordinate for a transfer.
 * @field data.xfer.size the file size for a transfer.
 * @field data.xfer.type the file type for a transfer.
 * @field data.xfer.name the file name for a transfer.
 * @note This is a bit more limited that RISC_OSLib's WIMP message block, but
 * it does for now. If extensions are required, then do not change the overall
 * structure of this - extend to the data union.
 */
typedef struct {
  int size;
  int sender;
  int my_ref;
  int your_ref;
  int action;
  union {
    char chars[232];
    int	words[58];
    struct {
      int w;
      int i;
      int x;
      int y;
      int size;
      int type;
      char name[212];
    } xfer;
  } data;
} message_block;

/**
 * Structure of a raw mouse information block.
 * @field x the (screen) x coordinate.
 * @field y the (screen) y coordinate.
 * @field b the mouse buttons.
 * @field w the WIMP window handle.
 * @field i the WIMP icon handle.
 */
typedef struct {
  int x;
  int y;
  int b;
  int w;
  int i;
} main_mousestr;

/**
 * Initialise the ROV core system, and start as an application.
 * @param task_name the name of the application.
 * @return the WIMP task handle for this application.
 * @see main_taskhandle
 * @see main_initnormalloc()
 */
int main_init(char *task_name);

/**
 * Initialise the ROV core system, but do not initialise the memory
 * allocation system. It is used when flex is being used without dynamic
 * areas.
 * @param task_name the name of the application.
 * @return the WIMP task handle for this application.
 * @see flex.h
 * @see main_taskhandle
 * @see main_init()
 */
int main_initnormalloc(char *task_name);

/**
 * Quit
 * @note This is not absolutely necessary, as ROVLib adds this as an 
 * atexit() handler.
 */
void main_exit(void);

/**
 * Enable a special 'nocrash' operation, but storing stack information,
 * and registering handlers for aborts of various types. If an abort
 * occurs, then a message box appears, prompting the user to either try
 * to continue, or to actually abort the program. If the user elects to
 * continue, then execution continues at the point after this function.
 * @note Although it works fine, this isn't the most ideal system.
 * @note This could be extended to perform some find of once-off execption
 * system.
 */
void main_nocrash(void);

#ifdef OLDSTYLE
void * main_addhandler(int, void *);
#else
/**
 * Add a handler to an event handler.
 * @param event_code the event handler code.
 * @param ... the pointer to the event handler function.
 * @return the pointer to the old handler function.
 * @note You should not use this, unless you intend to override ROV's
 * functions, or if you wish to handle an event that ROV does not
 * handle by default.
 * @note You must not claim the HANDLER_RESERVED1 event.
 * @note You should not claim the HANDLER_MESSAGE, HANDLER_MESSAGEACK
 * note the HANDLER_ACKNOWLEDGE - you should use the functions in
 * handextend.h instead.
 * @see Handler event codes
 * @see handextend.h
 */
void * main_addhandler(int event_code,
		       ...);
#endif

/**
 * Acknowledge a message
 * @param block the (modified) message block to acknowledge.
 * @param reason the WIMP message reason to reply with (17, 18 or 19).
 * @param action the action code to use.
 * @note my_ref and your_ref are swapped for you.
 * @see main_send()
 * @see message_block
 */
void main_acknowledge(message_block *block,
		      int reason,
		      int action);

/**
 * Send a message
 * @param block the message block to send.
 * @param reason the WIMP message reason to send.
 * @param to either the task handle, or the WIMP window handle of
 * the destination.
 * @param ic the WIMP icon handle, if to is a WIMP window handle.
 * @return the destination task handle.
 */
int main_send(message_block *block,
	      int reason,
	      int to,
	      int ic);

/**
 * Transfer a block of memory
 * @param src_task the source task handle.
 * @param src_ptr the source memory pointer.
 * @param dst_task the destination task handle.
 * @param dst_ptr the destination memory pointer.
 * @param len the transfer length.
 */
void main_transferblock(int src_task,
			void *src_ptr,
			int dst_task,
			void *dst_ptr,
			int len);

/**
 * Never-ending poll loop.
 * @note Unlike RISC_OSLib where you call their poll within a loop, this
 * one never terminates.
 */
void main_poll(void);

/**
 * Set the polling pollword pointer
 * @param ptr the pointer.
 */
void main_setpollword(void *ptr);

/**
 * Polls repeatedly until the first null event.
 */
void poll_once(void);

/**
 * Makes the next poll explicitly null pollable.
 */
void main_nextnull(int);

/**
 * Gets the OS monotonic timer.
 * @return the OS_ReadMonotonicTime value.
 */
int main_time(void);

/**
 * The message block used for polling
 */
extern message_block poll_block;

/**
 * The last poll reason code.
 */
extern int poll_lastevent;

/**
 * Initialise a different memory allocation system if flex
 * without dynamic areas is being used.
 * @see flex.h
 * @see main_initnormalloc()
 */
void rmalloc_newinit(void);

/**
 * Allocate memory.
 * @param len the amount to allocate.
 * @return the pointer to the memory, or NULL if unable to allocate.
 * @note Equivalent to malloc().
 */
void *rmalloc(size_t len);

/** 
 * Allocate memory.
 * @param len the size of the object.
 * @param count the number of objects.
 * @return the pointer to the memory, or NULL if unable to allocate.
 * @note Equivalent to calloc(), except memory is not zero'd
 */
void *rcalloc(size_t len,
	      int count);

/**
 * Free memory.
 * @param ptr the pointer to free.
 * @note Equivalent to free().
 */
void rfree(void *ptr);

/**
 * Produce a simple warning
 * @param str the string to display.
 * @param flags OK and cancel flags.
 * @return 0 for nothing clicked, 1 for OK clicked or 2 for Cancel clicked.
 * @note This simply calls Wimp_ReportError.
 * @see Simple warning flags
 * @see Simple warning responses
 */
int simple_warning(char *str,
		   int flags);


/**
 * Get the state of the 3D enabled flag in CMOS.
 * @return non-zero if 3D is enabled.
 */
int get_3dstatus(void);

/**
 * Get the state of the DragASprite enabled in CMOS.
 * @return non-zero if DragASprite is enabled.
 */
int get_dragstatus(void);

/**
 * Produce a new-style error box on any version of RISC OS.
 * @param error pointer to RISC OS error.
 * @param title the title of the error box.
 * @param sprite the name of the sprite on left hand side.
 * @param nboxes the number of boxes.
 * @param ... list of pointers to strings for each box name.
 * @return index of the box clicked on.
 * @note This does not use RISC OS 3.5+ error boxes, and performs its own display
 * of an error-style box that works on any version of RISC OS.
 * @see new_errorbox()
 * @see ro35error.h
 * @see QUIT
 */
int new_errorbox(void *error,
		 char *title,
		 char *sprite,
		 int nboxes,
		 ...);

/**
 * Produce a new-style error box on any version of RISC OS.
 * @param text pointer to the text that appears as the error.
 * @param title the title of the error box.
 * @param sprite the name of the sprite on left hand side.
 * @param nboxes the number of boxes.
 * @param ... list of pointers to strings for each box name.
 * @return index of the box clicked on.
 * @note This differs from new_errorbox() in that it can be passed a pure text
 * string to display.
 * @see new_errorbox()
 * @see ro35error.h
 * @see QUIT
 */
int new_errorboxtxt(char *text,
		    char *title,
		    char *sprite,
		    int nboxes,
		    ...);

/**
 * Makes the new errorbox functions 3D instead of 2D.
 * @note This is deprecated - it is called internally.
 */
void errorbox_3d(void);

/**
 * Get the OS version (from OS_Byte 129, 0, 255)
 * @return the OS version number.
 */
int main_getosversion(void);

/**
 * Get the raw mouse coordinates and button state
 * @param xp the pointer to hold the (screen) x coordinate.
 * @param yp the pointer to hold the (screen) y coordinate.
 * @param bp the pointer to hold the button state.
 */
void main_getrawmouse(int *xp,
		      int *yp,
		      int *bp);

/**
 * Get the machine type.
 * @return the machine type.
 * @see Machine types
 */
int main_machinetype(void);

/**
 * 'Send' a key to the WIMP (to deliver an unprocessed key)
 * @param chcode the character code to send.
 * @note This calls Wimp_ProcessKey.
 */
void main_processkey(int chcode);

/**
 * Start a new task.
 * @param exec the task's execution name.
 * @return the created task's task handle.
 * @note this calls Wimp_StartTask.
 */
int main_starttask(char *exec);

/**
 * Get the total memory size.
 * @return the total memory size in bytes.
 */
int main_getmemorysize(void);

/**
 * Get length of ctrl terminated string.
 * @param str the string to get the length of.
 * @note This is similar to strlen(), except that the string can be
 * ctrl terminated.
 */
int main_strlen(char *str);

/**
 * Copy a ctrl terminated string.
 * @param dst the destination.
 * @param src the source.
 * @note This is similar to strcpy(), except that the source string
 * can be ctrl terminated.
 * @note The destination string will be zero terminated.
 */
void main_strcpy(char *dst,
		 char *src);

/**
 * Compare two ctrl terminated strings.
 * @param str1 the first string.
 * @param str2 the second string.
 * @return negative if str1 < str2, zero if str1 = str2, and positive
 * if str1 > str2.
 * @note This is similar to strcmp(), except that the strings can be
 * ctrl terminated.
 */
int main_strcmp(char *str1,
		char *str2);

/**
 * Get the pixel width of a string.
 * @param str the string to get the width of.
 * @return the string width in OS units.
 * @note If the WIMP is using a font, then this is the width of the
 * string in that font. If the WIMP is not using a font (or is
 * incapable of doing so), then it is the width in OS characters.
 */
int main_strwidth(char *str);

/**
 * Wait for a vertical sync.
 * @note This calls OS_Byte 19.
 */
void main_vsync(void);

/**
 * Get the current date from the real-time clock.
 * @param date the block to store the date in.
 * @note This is identical to file_getcurrentdate() in file.h.
 * @see file.h
 */
void main_getdate(date_str *date);

/**
 * The text on the "Quit" icon for error boxes.
 */
extern char QUIT[];

/**
 * The text on the "Cancel" icon for error boxes.
 */
extern char CANCEL[];

/**
 * The text on the "Continue" icon for error boxes.
 */
extern char CONTINUE[];

/**
 * The text on the "OK" icon for error boxes.
 */
extern char OK[];

/**
 * The sprite name for error boxes.
 */
extern char ERRORSPRITE[];

/**
 * The task name for error boxes.
 */
extern char TASKNAME[];

/**
 * The text on the "Apply" icon for error boxes.
 */
extern char APPLY[];

/**
 * The text on the "Save" icon for error boxes.
 */
extern char SAVE[];

/**
 * The WIMP version when called from Wimp_Initialise.
 */
extern int main_wimpversion;

/**
 * The task handle assigned to this application.
 */
extern int main_taskhandle;

/**
 * @section Handler event codes
 */
#define HANDLER_NULL		(0)
#define HANDLER_REDRAW		(1)
#define HANDLER_OPEN		(2)
#define HANDLER_CLOSE		(3)
#define HANDLER_ENTERWINDOW	(4)
#define HANDLER_LEAVEWINDOW	(5)
#define HANDLER_MOUSE		(6)
#define HANDLER_DRAG		(7)
#define HANDLER_KEY		(8)
#define HANDLER_MENU		(9)
#define HANDLER_SCROLL		(10)
#define HANDLER_LOSECARET	(11)
#define HANDLER_GAINCARET	(12)
#define HANDLER_MESSAGE		(13)
#define HANDLER_MESSAGEACK	(14)
#define HANDLER_ACKNOWLEDGE	(15)
#define HANDLER_SAVE		(16)
#define HANDLER_SAVEACK		(17)
#define HANDLER_LOAD		(18)
#define HANDLER_LOADACK		(19)
#define HANDLER_RUN		(20)
#define HANDLER_HELP		(21)
#define HANDLER_MODE		(22)
#define HANDLER_PALETTE		(23)
#define HANDLER_PREQUIT		(24)
#define HANDLER_MENUWARNING	(25)
#define HANDLER_RESERVED1       (26)
#define HANDLER_RMALLOC         (27)
#define HANDLER_RCALLOC         (28)
#define HANDLER_RFREE           (29)
#define HANDLER_CLIPCLAIM	(30)
#define HANDLER_CLIPREQUEST	(31)
#define HANDLER_DRAGGING	(32)
#define HANDLER_DRAGCLAIM	(33)
#define HANDLER_RAMFETCH	(34)
#define HANDLER_RAMTRANSMIT	(35)
#define HANDLER_CLAIMDEVICE	(36)
#define HANDLER_DEVICEUNCLAIMED	(37)
#define HANDLER_DEVICECLAIMED	(38)
#define HANDLER_TASKEXIT	(39)

/**
 * @section Simple warning flags
 */
#define WARN_OK			(0)
#define WARN_CANCEL		(1)
#define WARN_OKCANCEL		(2)

/**
 * @section Simple warning responses
 */
#define WARN_NOCLICK		(0)
#define WARN_OKSELECTED		(1)
#define WARN_CANCELSELECTED	(2)

/**
 * @section WIMP messages
 */
#define MESSAGE_QUIT		(0)
#define MESSAGE_DATASAVE	(1)
#define MESSAGE_DATASAVEACK	(2)
#define MESSAGE_DATALOAD	(3)
#define MESSAGE_DATALOADACK	(4)
#define MESSAGE_DATAOPEN	(5)
#define MESSAGE_DATARAMFETCH	(6)
#define MESSAGE_DATARAMTRANSMIT	(7)
#define MESSAGE_PREQUIT		(8)
#define MESSAGE_PALETTECHANGED	(9)
#define MESSAGE_CLAIMENTITY	(15)
#define MESSAGE_DATAREQUEST	(16)
#define MESSAGE_DRAGGING	(17)
#define MESSAGE_DRAGCLAIM	(18)

/**
 * @section Button types
 */
#define BUTTON_RIGHT            (1)
#define BUTTON_MENU             (2)
#define BUTTON_LEFT             (4)
#define DRAG_RIGHT              (16)
#define DRAG_LEFT               (64)
#define CLICK_RIGHT             (256)
#define CLICK_LEFT              (1024)

/**
 * @section Machine types
 */
#define MACHINE_PRE5000         (0)
#define MACHINE_PRERISCPC	(1)
#define MACHINE_RISCPCSTYLE	(2)
#define MACHINE_DOESNOTEXIST	(128)
#define MACHINE_A4		(129)
#define MACHINE_RISCPCLITE	(130)

/* Format of handlers :

handler(int code, < handler_specific_data > * data);

 */


#ifdef __cplusplus
}
#endif

#endif
